"use strict";
exports.__esModule = true;
var child_process_1 = require("child_process");
var path = require("path");
var util = require("util");
Object.prototype.dump = function () {
    return util.inspect(this, {
        depth: null,
        maxArrayLength: null,
        breakLength: 0
    });
};
String.prototype.camelCase = function () {
    return this.toLowerCase()
        .replace(/-(.)/g, function (match, group1) {
        return group1.toUpperCase();
    });
};
String.prototype.underscore = function () {
    return this.replace(/([A-Z])/g, function (m) { return '_' + m.toLowerCase(); });
};
function asyncWriter(stream) {
    var draining = true;
    function doWrite(chunk) {
        return new Promise(function (resolve, reject) {
            if (draining) {
                draining = stream.write(chunk, function (err) {
                    if (err) {
                        reject(err);
                    }
                    else {
                        resolve();
                    }
                });
            }
            else {
                stream.once('drain', function () {
                    // await recursive
                    doWrite(chunk)
                        .then(resolve)["catch"](reject);
                });
            }
        });
    }
    return doWrite;
}
exports.asyncWriter = asyncWriter;
function indexById(types) {
    var ret = {};
    types.forEach(function (type) {
        ret[type.sys.id] = type;
    });
    return ret;
}
exports.indexById = indexById;
function wait(ms) {
    return new Promise(function (resolve, reject) {
        setTimeout(function () { return resolve(); }, ms);
    });
}
exports.wait = wait;
function formatFile(file) {
    var tsFmtBinLocation = path.join(require.resolve('typescript-formatter'), '../../.bin/tsfmt');
    var tsfmtConfigFile = path.relative(process.cwd(), path.join(__dirname, '../tsfmt.json'));
    return new Promise(function (resolve, reject) {
        child_process_1.exec(tsFmtBinLocation + " -r " + file + " --useTsfmt " + tsfmtConfigFile, function (err, stdout, stderr) {
            if (err) {
                reject(err.message + '\n\t' + stderr);
            }
            else {
                resolve();
            }
        });
    });
}
exports.formatFile = formatFile;
